#include <iostream>
#include <thread>
#include <string>
#include <iomanip>

#include "tools.h"

#include "naive.hpp"
#include "ring.hpp"

// #include "student.hpp"

template< template < typename > class Queue >
void bench( std::string name ) {

    static const int items = 4 * 16 * 1024 * 1024;
    Queue< int > queue;
    Timer timer;

    timer.start();

    std::thread writer( [&] {
        int n = 0;
        while ( n < items )
            queue.enqueue( ++n );
    } );
    std::thread reader( [&] {
        int n = 0, o = 0;
        while ( n < items ) {
            if ( !queue.empty() ) {
                int o = queue.dequeue();
                assert_eq( n + 1, o );
                n = o;
            }
        }
    } );

    writer.join();
    reader.join();

    timer.stop();

    std::cout << std::setw( 20 ) << name << ": " << 
        std::setw( 10 ) << timer.elapsed() << " ms, " <<
        std::setw( 6 ) << ( items / timer.elapsed() ) << " items/ms." << std::endl;

}

int main() {
    bench< Naive >( "naive" );

    bench< Ring<  512 >::Q >( "ring 512" );
    bench< Ring< 1024 >::Q >( "ring 1024" );
    bench< Ring< 4096 >::Q >( "ring 4096" );

    // TODO: insert benchmarks for your implementation(s) here
    
    return 0;
}