#pragma once

#include <chrono>
#include <iostream>

#include "compatibilityHeader.h"

class Timer {
    using TimePoint = std::chrono::time_point < std::chrono::system_clock >;
    TimePoint _startTime;
    TimePoint _stopTime;
public:
    void start() {
        _startTime = now();
    }

    void stop() {
        _stopTime = now();
    }

    TimePoint now() {
        return std::chrono::system_clock::now();
    }

    long long elapsed() const {
        return std::chrono::duration_cast< std::chrono::milliseconds >( _stopTime - _startTime ).count();
    }
};

namespace _internal {
    class Location {
        const char *_file;
        int _line;
        const char *_fmtMessage;
    public:
        Location( const char *file, int line, const char *fm ) :
            _file( file ),
            _line( line ),
            _fmtMessage( fm ) {}
        const char *file() const {
            return _file;
        }
        int line() const {
            return _line;
        }
        const char *message() const {
            return _fmtMessage;
        }
    };

    template< typename X, typename Y >
    void assert_eq_fn( X x, Y y, const Location &location ) {
        std::cerr << location.file() << ": " << location.line() <<
            ": assertion `" << location.message() << "' failed;" <<
            " got [" << x << "] != [" << y << "] instead";
        std::abort();
    }

}

#define assert_eq( x, y )                                                           \
    do {                                                                            \
        using namespace ::_internal;                                                \
        if ( !( x == y ) )                                                          \
            assert_eq_fn( x, y, Location( __FILE__, __LINE__, #x " == " #y ) );     \
    } while ( false )

